from typing import Awaitable

import anyio


def run_sync(func: Awaitable, *args, **kwds):
    async def executor():
        return await func(*args, **kwds)

    return anyio.from_thread.run(executor)



from fastack.globals import LocalProxy, state

from fastack_mail.backend import EmailBackend


def _get_email_backend() -> EmailBackend:
    backend = getattr(state, "email", None)
    if not isinstance(backend, EmailBackend):
        raise RuntimeError("fastack-mail is not installed")
    return backend


email: EmailBackend = LocalProxy(_get_email_backend)



from typing import Awaitable, Iterable, Optional, Sequence, Union

from aiosmtplib import SMTP
from aiosmtplib.connection import DEFAULT_TIMEOUT

from fastack_mail.message import Message
from fastack_mail.sync import run_sync


class EmailBackend:
    def __init__(
        self,
        *,
        hostname: Optional[str] = "localhost",
        port: Optional[int] = None,
        username: Optional[Union[str, bytes]] = None,
        password: Optional[Union[str, bytes]] = None,
        timeout: Optional[float] = DEFAULT_TIMEOUT,
        use_tls: bool = False,
        start_tls: bool = False,
        validate_certs: bool = True,
        client_cert: Optional[str] = None,
        client_key: Optional[str] = None,
        cert_bundle: Optional[str] = None,
        async_mode: bool = True,
    ) -> None:
        self.hostname = hostname
        self.port = port
        self.username = username
        self.password = password
        self.timeout = timeout
        self.use_tls = use_tls
        self.start_tls = start_tls
        self.validate_certs = validate_certs
        self.client_cert = client_cert
        self.client_key = client_key
        self.cert_bundle = cert_bundle
        self._client: Optional[SMTP] = None
        self._async_mode = async_mode

    @property
    def client(self) -> SMTP:
        if self._client is None:
            self._client = SMTP(
                hostname=self.hostname,
                port=self.port,
                username=self.username,
                password=self.password,
                timeout=self.timeout,
                use_tls=self.use_tls,
                start_tls=self.start_tls,
                validate_certs=self.validate_certs,
                client_cert=self.client_cert,
                client_key=self.client_key,
                cert_bundle=self.cert_bundle,
            )
        return self._client

    def send(
        self,
        message: Message,
        *,
        sender: Optional[str] = None,
        recipients: Optional[Union[str, Sequence[str]]] = None,
        mail_options: Optional[Iterable[str]] = None,
        rcpt_options: Optional[Iterable[str]] = None,
    ) -> Optional[Awaitable]:
        assert isinstance(
            message, Message
        ), "Message must be an instance of fastack_mail.message.Message"
        msg = message.build()
        params = {
            "sender": sender,
            "recipients": recipients,
            "mail_options": mail_options,
            "rcpt_options": rcpt_options,
        }

        async def func():
            async with self.client:
                return await self.client.send_message(msg, **params)

        if self._async_mode:
            return func()
        return run_sync(func)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastack_mail']

package_data = \
{'': ['*']}

install_requires = \
['aiosmtplib>=1.1.6,<2.0.0',
 'fastack>=4.0.0,<5.0.0',
 'pydantic[email]>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'fastack-mail',
    'version': '0.1.0',
    'description': 'Simple plugin for sending email',
    'long_description': '# fastack-mail\n\nSimple plugin for sending email.\n\nThis plugin is inspired by [fastapi-mail](https://github.com/sabuhish/fastapi-mail)\n\n## Installation\n\n```\npip install fastack-mail\n```\n\n## Usage\n\n### Install plugin\n\n```py\nPLUGINS = [\n    ...,\n    "fastack_mail",\n]\n```\n\n### Configuration\n\n* `EMAIL_BACKEND` = "fastack_mail.backend.EmailBackend" [required]\n* `EMAIL_HOSTNAME` = SMTP Server (e.g. "localhost") [required]\n* `EMAIL_PORT` = SMTP Port Server (e.g. 8025) [required]\n* `EMAIL_USERNAME` = SMTP username (e.g. "john@doe.com") [optional]\n* `EMAIL_PASSWORD` = SMTP password (e.g. "luarbiasa") [optional]\n* `EMAIL_TIMEOUT` = Connection time out (default 60) [optional]\n* `EMAIL_USE_TLS` = If `True`, make the _initial_ connection to the server over TLS/SSL (default False)\n* `EMAIL_START_TLS` = If `True`, make the _initial_ connection to the server over plaintext, and then upgrade the connection to TLS/SSL. Not compatible with `EMAIL_USE_TLS`. (default False)\n* `EMAIL_VALIDATE_CERTS` = Determines if server certificates are validated (default True)\n* `EMAIL_CLIENT_CERT` = Path to client side certificate, for TLS verification. (default None)\n* `EMAIL_CLIENT_KEY` = Path to client side key, for TLS verification. (default None)\n* `EMAIL_CERT_BUNDLE` = Path to certificate bundle, for TLS verification. (default None)\n* `EMAIL_ASYNC_MODE` = If `True`, the `EmailBackend.send` function returns an awaitable object, if `False` it returns an immediate send result. (default False)\n* `DEFAULT_FROM_EMAIL` = Default sender email for use globally (if there is no sender email) (e.g. "Local \\<noreply@localhost\\>")\n\n### Example\n\nSee here https://github.com/fastack-dev/fastack-mail/tree/main/examples/mail\n',
    'author': 'aprilahijriyan',
    'author_email': 'hijriyan23@gmail.com',
    'maintainer': 'aprilahijriyan',
    'maintainer_email': '37798612+aprilahijriyan@users.noreply.github.com',
    'url': 'https://github.com/fastack-dev/fastack-mail',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<3.10',
}


setup(**setup_kwargs)



from fastack import Fastack
from fastack.utils import import_attr

from fastack_mail.backend import EmailBackend


def setup(app: Fastack):
    params = {}
    prefix = "EMAIL_"
    email_backend_class_setting = prefix + "BACKEND"
    for setting in dir(app.state.settings):
        if not setting.startswith(prefix) or setting == email_backend_class_setting:
            continue

        value = app.get_setting(setting)
        setting = setting[len(prefix) :].lower()
        params[setting] = value

    klass = app.get_setting(email_backend_class_setting)
    try:
        email_backend_class = import_attr(klass)
    except Exception as e:
        raise RuntimeError(f"Unable to import email backend {klass}") from e

    email_backend: EmailBackend = email_backend_class(**params)
    app.state.email = email_backend
